# --------------------------------------------------------------
# Applied Statistics / Statistical methods for the Biosciences
# Day 3: Categorical regression
# Solution to Exercise 3.4
# Bo Markussen
# November 29, 2017
# -------------------------------------------------------------

# Load libraries we will be using
library(ordinal)

# Read data
walks <- read.delim("walks.txt")
str(walks)

# The following call shows, that "frequency" is read in as a character string.
class(walks$frequency)

# As is the case for glm(), does clm() not accept a character variable as the response.
# So we recode "frequency" to a categorical factor
walks$frequency <- factor(walks$frequency)
class(walks$frequency)

# The variable "distance" is numerical
class(walks$distance)

# However, in the statistical analysis below we will both use "distance" as a numerical
# variable, and as a categorical variable (with levels: 1, 3.5, 7.5, 15, 25). So 
# we will not change the variable type of "distance", but instead write "factor(distance)"
# whenever we want to use it as a categorical variable.


# ----------------------
# Distance on frequency
# ----------------------

# Lack-of-Fit for proportional odds assumption
m0 <- clm(factor(distance)~1,nominal=~frequency,data=walks,weights=count)
m1 <- clm(factor(distance)~frequency,data=walks,weights=count)
anova(m1,m0)

# The proportional odds assumption is valid, and we proceed with effect testing
drop1(m1,test="Chisq")

# Effect of frequency is highly significant. 
# We proceed by giving estimates for odds ratios (for walking shorter distances)
exp(cbind("OR vs F1"=coef(m1)[5:6],confint(m1)))


# ----------------------
# Frequency on distance
# ----------------------

# Lack-of-Fit for proportional odds assumption
m0 <- clm(frequency~1,nominal=~factor(walks$distance),data=walks,weights=count)
m1 <- clm(frequency~factor(distance),data=walks,weights=count)
anova(m1,m0)

# The proportional odds assumption is valid, and we proceed with effect testing
drop1(m1,test="Chisq")

# Effect of distance is highly significant.
# Let's see if the categorical distance may be used as a continuous variable instead:
m2 <- clm(frequency~distance,data=walks,weights=count)
anova(m2,m1)

# It was not particularly successful to use distance as a continuous variable;
# this namely gave a larger AIC value. And hence a worse model.
# But perhaps it is better to use distance of a logarithmic scale; 
# this wouldn't be surprising considering the "logarithmic structure" of the 
# proportional odds model.
m3 <- clm(factor(frequency)~log(distance),data=walks,weights=count)
anova(m3,m1)

# The null hypothesis that distance may be used as a continuous covariate
# on the logarithmic scale gave an improved model!!! 
# We proceed by testing the effect of distance as a continuous variable:
drop1(m3,test="Chisq")

# Finally we give the estimated odds ratio (for walking less frequent)
# when the person usually walks twice as long:
# Additional question: Why do we multiply by log(2) before taking the exponential?
exp(log(2)*cbind(estimate=coef(m3)[3],confint(m3)))

# Remark: The variable 'distance' is more significant when used as a continuous 
# variable instead as a categorical factor. The likelihood ratio test statistics
# (LRT in the output from drop1()) are almost the same, but the degrees of freedom 
# is only 1 when 'distance' is use as a continuous variable. However, in this particular 
# situation this is not important as both tests are highly significant.
